unit WPRCMMTimer;

{ Copyright (c) 2007 tadashi.ikoma, All rights reserved. }
{ Interval ́@ŏlAőlm肽ꍇ́A
vOŁAMin,Max̒lǂݏoĂB}
{\ Resolution ̒ǉ TimeGetTime ̕\ɂêŒӂ鎖}

interface

uses
  SysUtils, WinTypes, Classes, MMSystem, Forms, Messages, Windows;

type

  EMMTimerError = class(Exception);
  TWPRCMMTimer = class(TComponent)
  private
    FID: Cardinal;
    FInterval: Cardinal;
    FResolution: Cardinal;
    FOnTimer: TNotifyEvent;
    FEnabled: Boolean;
    TMinMax: TIMECAPS;
    FWindowHandle:HWND;                               // WndProcd Handle
    procedure UpdateTimer;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure SetResolution(Value: Cardinal);
    procedure SetOnTimer(Value:TNotifyEvent);
  protected
    procedure Timer; dynamic;
    procedure Loaded; override;
    procedure WndProcd(var Msg:TMessage);             // TChart@sI^[h~
  public
    Min: Cardinal;
    Max: Cardinal;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property Resolution: Cardinal read FResolution write SetResolution default 1;
    property OnTimer: TNotifyEvent read FOnTimer write SetOnTimer;
  end;

procedure Register;

implementation

uses ShellAPI, Registry;


procedure MMTimerProc(ID, Reserved: Cardinal;
                      dwUser, dwReserved1, dwReserved2: Nativeuint); stdcall;
begin
  SendMessage(dwUser, WM_TIMER, 0, 0);
end;

constructor TWPRCMMTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWindowHandle := AllocateHWnd(WndProcd);
  FEnabled := True;
  FInterval := 1000;
//^C}[foCX̐\擾
  TimeGetDevCaps(@TMinMax,sizeof(TMinMax));
  Min := TMinMax.wPeriodMin;
  Max := TMinMax.wPeriodMax;
  FResolution := Min;
end;

procedure TWPRCMMTimer.Loaded;
begin
  inherited Loaded;
  UpdateTimer;
end;

destructor TWPRCMMTimer.Destroy;
begin
  FEnabled := False;
  UpdateTimer;
  DeallocateHWnd(FWindowHandle);            // WndProcd Handle ̔j
  inherited Destroy;
end;

procedure TWPRCMMTimer.UpdateTimer;
begin
  if FID <> 0 then timeKillEvent(FID);
  if FEnabled and Assigned(FOnTimer) then
    begin
      if FInterval >= FResolution then
        begin
          FID := timeSetEvent(FInterval, FResolution, MMTimerProc, FWindowHandle, TIME_PERIODIC);
          if FID = 0 then
            begin
              FEnabled := False;
              raise EMMTimerError.Create('No MMTimer Available');
            end;
        end
        else
        begin
          FEnabled := False;
          raise EMMTimerError.Create('a Large Resolution of MMTimer');
        end;
  end;
end;

procedure TWPRCMMTimer.WndProcd(var Msg:TMessage);
begin
  with Msg do
    case Msg of
      WM_TIMER:
        try
          Timer;
        except
          Application.HandleException(Self);
        end;
      else
        Result := DefWindowProc(FWindowHandle, Msg, WParam, LParam);
    end;
end;

procedure TWPRCMMTimer.Timer;
begin
  if Assigned(FOnTimer) then FOnTimer(Self);
end;

procedure TWPRCMMTimer.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
    begin
      FEnabled := Value;
      UpdateTimer;
    end;
end;

procedure TWPRCMMTimer.SetInterval(Value: Cardinal);
begin
  if FInterval <> Value then
    begin
//^C}[͈̔͂𒴂Đݒ肵AG[\
      if Min > Value then
        begin
          raise EMMTimerError.Create('time under' + #13#10 +
                                        'minimum '+ intTostr(Min));
          exit;
        end;
      if Max < Value then
        begin
          raise EMMTimerError.Create('time over' + #13#10 +
                                        'maximum ' + intTostr(Max));
          exit;
        end;
      FInterval := Value;
      UpdateTimer;
    end;
end;

procedure TWPRCMMTimer.SetResolution(Value: Cardinal);
begin
  if FResolution <> Value then
    begin
//^C}[̕\𒴂Đݒ肵A^C}[\C
      if Min > Value then FResolution := Min
                     else FResolution := Value;
      UpdateTimer;
    end;
end;

procedure TWPRCMMTimer.SetOnTimer(Value:TNotifyEvent);
begin
  FOnTimer := value;
  UpdateTimer;
end;

procedure Register;
begin
  RegisterComponents('MMTimer', [TWPRCMMTimer]);
end;

end.
